/***************************************************************************
    NWNXArray.cpp - Implementation of the CNWNXODBC class.
    Copyright (C) 2003 Ingmar Stieger (Papillon, papillon@blackdagger.com) and 
	Jeroen Broekhuizen (nwnx@jengine.nl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ***************************************************************************/

#include "stdafx.h"
#include "NWNXOdbc.h"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CNWNXODBC::CNWNXODBC()
{
	request_counter = 0;
	sqlerror_counter = 0;
}

CNWNXODBC::~CNWNXODBC()
{
	OnRelease ();
}

BOOL CNWNXODBC::OnCreate (const char* LogDir)
{
	// call the base class function
	char log[MAX_PATH];
	sprintf (log, "%s\\nwnx_odbc.txt", LogDir);
	if (!CNWNXBase::OnCreate(log))
		return FALSE;

	// connect to the database
	return Connect ();
}

char* CNWNXODBC::OnRequest (char* gameObject, char* Request, char* Parameters)
{
	if (strncmp (Request, "EXEC", 4) == 0)
		Execute (Parameters, strlen (Parameters));
	if (strncmp (Request, "FETCH", 5) == 0)
		Fetch (Parameters, strlen (Parameters));

	return NULL;
}

BOOL CNWNXODBC::OnRelease ()
{
	Log ("o Disconnecting from database.\n");
	//m_ODBC.Disconnect ();

	// call base class function
	return CNWNXBase::OnRelease ();
}


BOOL CNWNXODBC::Connect()
{
	// try to connect to the database
	if (!m_ODBC.Connect ())
	{
		Log ("! Error while connecting to database: %s\n", m_ODBC.GetLastError ());
		return FALSE;
	}

	// we successfully connected
	Log ("o Connect successfull.\n");
	return TRUE;
}

void CNWNXODBC::Execute(char *request, unsigned int responseSize)
{
	Log ("o Got request: %s\n", request);
	request_counter++;

	// try to execute the SQL query
	if (!m_ODBC.ExecuteSQL (request))
		Log ("! SQL Error: %s\n", m_ODBC.GetLastError ());
}

void CNWNXODBC::Fetch(char *buffer, unsigned int buffersize)
{
	unsigned int totalbytes = 0;
	buffer[0] = 0x0;

	// fetch data from recordset
	totalbytes = m_ODBC.Fetch (buffer, buffersize);
	if (buffersize == -1)
		Log ("o Empty set\n");

	// log what we received
	Log ("o Sent response (%d bytes): %s\n", totalbytes, buffer);
}
