/***************************************************************************
    ODBC1.cpp - Implementation of the CODBC class.
    Copyright (C) 2003 Ingmar Stieger (Papillon, papillon@blackdagger.com) and 
	Jeroen Broekhuizen (nwnx@jengine.nl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ***************************************************************************/

#include "stdafx.h"
#include "ODBC1.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CODBC::CODBC()
{
	henv = NULL;
	hdbc = NULL;
	hstmt = NULL;
}

CODBC::~CODBC()
{

}

BOOL CODBC::Connect(char DSN[], int timeout)
{
	SQLAllocEnv (&henv);
	SQLAllocConnect (henv,&hdbc);
	SQLSetConnectAttr (hdbc, SQL_LOGIN_TIMEOUT, &timeout, 0);
	rc = SQLConnect (hdbc, (unsigned char*)DSN, SQL_NTS, NULL, 0, NULL, 0);

	// On error: deallocate handles, display error message, and exit.
	if (!MYSQLSUCCESS(rc))
	{
		SQLCHAR SqlState[6];
		SQLINTEGER NativeError;
		SQLSMALLINT MsgLen;

		SQLGetDiagRec(SQL_HANDLE_DBC, hdbc, 1, SqlState, &NativeError, (unsigned char *)lastError, 2048, &MsgLen);
		SQLFreeEnv(henv);
		SQLFreeConnect(hdbc);
		return FALSE;
	}

	// connecting was succesfull
	rc=SQLAllocStmt(hdbc, &hstmt);
	return TRUE;
}

void CODBC::Disconnect()
{
	// free up the ODBC connection
	SQLFreeStmt(hstmt, SQL_DROP);
	SQLDisconnect(hdbc);
	SQLFreeConnect(hdbc);
	SQLFreeEnv(henv);
}

BOOL CODBC::ExecuteSQL(char *query)
{
	SQLFreeStmt(hstmt,SQL_CLOSE);

	// execute the query
	rc=SQLExecDirect(hstmt, (unsigned char *)query, SQL_NTS);
	if (!MYSQLSUCCESS(rc))  // Error
	{
		SaveSQLError ();
		return FALSE;
	}
	return TRUE;
}

unsigned int CODBC::Fetch(char *buffer, unsigned int buffersize)
{
	unsigned int totalbytes = 0;
	SQLSMALLINT NumCols, i;

	buffer[0] = 0x0;

	// walk through the resultset
	if (SQLFetch(hstmt) != SQL_SUCCESS)
		return -1;

	// add each column to buffer
	SQLNumResultCols(hstmt, &NumCols);
	for (i = 1; i <= NumCols; i++)
	{
		SQLGetData(hstmt, i, SQL_C_CHAR, columnData, MAX_DATA, &cbData);
		
		if ((totalbytes + cbData <= buffersize) && (cbData != SQL_NULL_DATA))
		{
			totalbytes += cbData;					
			strcat(buffer, columnData);
		}

		if ((i != NumCols) && (totalbytes + 1 <= buffersize))
		{
			totalbytes++;
			strcat(buffer, "");  // ascii 170
		}
	}

	return totalbytes;
}

char* CODBC::GetLastError()
{
	// return last error
	return lastError;
}

void CODBC::SaveSQLError()
{
	unsigned char szSQLSTATE[10];
	SDWORD nErr;
	SWORD cbmsg;

	// fetch the last SQL error
	SQLError(0, 0, hstmt, szSQLSTATE, &nErr, (unsigned char*) lastError, sizeof(lastError), &cbmsg);
}

